Cassandra = LibStub("AceAddon-3.0"):NewAddon("Cassandra", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Cassandra")

-- begin default options
local defaults = {
    profile = {
        lootmoney = false,
        lootcurrency = false,
        lootquests = false,
        lootcrafting = false,
        lootquality = 4
    }
}
-- end default options

-- begin configuration options
local options = {
    name = L["Cassandra"],
    handler = Cassandra,
    type = "group",
    args = {
        lootoptions = {
            name = L["Loot Options"],
            desc = L["Configure looting options"],
            type = "group",
            args = {
                lootmoney = {
                    name = L["Loot Money"],
                    desc = L["Should Cassandra loot money?"],
                    type = "toggle",
                    set = "SetLootMoney",
                    get = "IsLootMoney"
                },
                lootcurrency = {
                    name = L["Loot Currency"],
                    desc = L["Should Cassandra loot currency?"],
                    type = "toggle",
                    set = "SetLootCurrency",
                    get = "IsLootCurrency"
                },
                lootquests = {
                    name = L["Loot Quest Items"],
                    desc = L["Should Cassandra loot quest items?"],
                    type = "toggle",
                    set = "SetLootQuests",
                    get = "IsLootQuests"
                },
                lootcrafting = {
                    name = L["Loot Crafting Mats"],
                    desc = L["Should Cassandra loot crafting materials?"],
                    type = "toggle",
                    set = "SetLootCrafting",
                    get = "IsLootCrafting"
                },
                lootquality = {
                    name = L["Loot Quality"],
                    desc = L["Cassandra should automatically loot items of this quality or better."],
                    type = "select",
                    set = "SetLootQuality",
                    get = "GetLootQuality",
                    values = {
                        "|cff9d9d9d" .. ITEM_QUALITY0_DESC,
                        "|cffffffff" .. ITEM_QUALITY1_DESC,
                        "|cff1eff00" .. ITEM_QUALITY2_DESC,
                        "|cff0070dd" .. ITEM_QUALITY3_DESC,
                        "|cffa335ee" .. ITEM_QUALITY4_DESC,
                        "|cffff8000" .. ITEM_QUALITY5_DESC,
                        "|cffe6cc80" .. ITEM_QUALITY6_DESC,
                        "|cffe6cc80" .. ITEM_QUALITY7_DESC,
                    },
                    style = "dropdown"
                }
            }
        }
    }
}
-- end configuration options

-- begin getters and setters
function Cassandra:SetLootMoney(info, newValue)
    self.db.profile.lootmoney = newValue
end

function Cassandra:IsLootMoney(info)
    return self.db.profile.lootmoney
end

function Cassandra:SetLootCurrency(info, newValue)
    self.db.profile.lootcurrency = newValue
end

function Cassandra:IsLootCurrency(info)
    return self.db.profile.lootcurrency
end

function Cassandra:SetLootQuests(info, newValue)
    self.db.profile.lootquests = newValue
end

function Cassandra:IsLootQuests(info)
    return self.db.profile.lootquests
end

function Cassandra:SetLootCrafting(info, newValue)
    self.db.profile.lootcrafting = newValue
end

function Cassandra:IsLootCrafting(info)
    return self.db.profile.lootcrafting
end

function Cassandra:SetLootQuality(info, newValue)
    self.db.profile.lootquality = newValue
end

function Cassandra:GetLootQuality(info)
    return self.db.profile.lootquality
end

-- end getters and setters

-- begin Ace3 functions
function Cassandra:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("CassandraDB", defaults, true)
    options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
    LibStub("AceConfig-3.0"):RegisterOptionsTable("Cassandra", options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Cassandra", "Cassandra")
end

function Cassandra:OnEnable()
    self:RegisterEvent("LOOT_OPENED")
end

-- end Ace3 functions

-- begin event handling
function Cassandra:LOOT_OPENED()
    local numLootItems = GetNumLootItems()
    for slot = 1, numLootItems do
        local lootIcon, lootName, lootQuantity, rarity = GetLootSlotInfo(slot)
        if lootQuantity == 0 then
            if self.db.profile.lootmoney then
                LootSlot(slot)
            end
        else
            local link = GetLootSlotLink(slot)
            if link ~= nil then
                local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType = GetItemInfo(link)
                if itemType == "Quest" then
                    if self.db.profile.lootquests then
                        LootSlot(slot)
                    end
                elseif itemType == "Money" then
                    if self.db.profile.lootcurrency then
                        LootSlot(slot)
                    end
                elseif itemType == "Trade Goods" then
                    if self.db.profile.lootcrafting then
                        LootSlot(slot)
                    end
                elseif rarity >= (self.db.profile.lootquality - 1) and rarity < GetLootThreshold() then
                    LootSlot(slot)
                end
            end
        end
    end
end

-- end event handling